
# NetBox Imports
from netbox.views import generic

# Proxbox Imports
from netbox_proxbox.models import ProxmoxEndpoint
from netbox_proxbox.tables import ProxmoxEndpointTable
from netbox_proxbox.filtersets import ProxmoxEndpointFilterSet
from netbox_proxbox.forms import ProxmoxEndpointForm, ProxmoxEndpointFilterForm


__all__ = (
    'ProxmoxEndpointView',
    'ProxmoxEndpointListView',
    'ProxmoxEndpointEditView',
    'ProxmoxEndpointDeleteView',
)


class ProxmoxEndpointView(generic.ObjectView):
    queryset = ProxmoxEndpoint.objects.all()


class ProxmoxEndpointListView(generic.ObjectListView):
    queryset = ProxmoxEndpoint.objects.all()
    table = ProxmoxEndpointTable
    filterset = ProxmoxEndpointFilterSet
    filterset_form = ProxmoxEndpointFilterForm

    
class ProxmoxEndpointEditView(generic.ObjectEditView):
    queryset = ProxmoxEndpoint.objects.all()
    form = ProxmoxEndpointForm


class ProxmoxEndpointDeleteView(generic.ObjectDeleteView):
    queryset = ProxmoxEndpoint.objects.all()


# Django Imports
from django.shortcuts import get_object_or_404
from django.urls import reverse

# NetBox Imports
from netbox.views import generic

# ProxBox Imports
from netbox_proxbox.models import FastAPIEndpoint
from netbox_proxbox.tables import FastAPIEndpointTable
from netbox_proxbox.filtersets import FastAPIEndpointFilterSet
from netbox_proxbox.forms import FastAPIEndpointForm, FastAPIEndpointFilterForm


__all__ = (
    'FastAPIEndpointView',
    'FastAPIEndpointListView',
    'FastAPIEndpointEditView',
    'FastAPIEndpointDeleteView',
)


class FastAPIEndpointView(generic.ObjectView):
    queryset = FastAPIEndpoint.objects.all()


class FastAPIEndpointListView(generic.ObjectListView):
    queryset = FastAPIEndpoint.objects.all()
    table = FastAPIEndpointTable
    filterset = FastAPIEndpointFilterSet
    filterset_form = FastAPIEndpointFilterForm


class FastAPIEndpointEditView(generic.ObjectEditView):
    """
    This view is used to edit and create the FastAPIEndpoint object.
    
    If there is already an existing FastAPIEndpoint object,
    the view will return the existing object, allowing only one object to be created.
    """
    template_name = 'netbox_proxbox/fastapiendpoint_edit.html'
    queryset = FastAPIEndpoint.objects.all()
    form = FastAPIEndpointForm
    
    def get_object(self, **kwargs):
        if int(FastAPIEndpoint.objects.count()) >= 1:
            return FastAPIEndpoint.objects.first()
    
        if not kwargs:
            # We're creating a new object
            return self.queryset.model()
        
        return get_object_or_404(FastAPIEndpoint.objects.all(), **kwargs)
    
    def get_extra_context(self, request, instance):
        if int(FastAPIEndpoint.objects.count()) >= 1:
            return {'existing_object': True,}
        
        return {'existing_object': False}


class FastAPIEndpointDeleteView(generic.ObjectDeleteView):
    queryset = FastAPIEndpoint.objects.all()
 


# Netbox plugin related import
from netbox.plugins import PluginConfig

class ProxboxConfig(PluginConfig):
    name = "netbox_proxbox"
    verbose_name = "Proxbox"
    description = "Integrates Proxmox and Netbox"
    version = "0.0.6b2"
    author = "Emerson Felipe (@emersonfelipesp)"
    author_email = "emersonfelipe.2003@gmail.com"
    min_version = '4.2.0'
    max_version = '4.2.99'
    base_url = "proxbox"
    required_settings = []

config = ProxboxConfig

#from . import proxbox_api


from django.views import View
from django.shortcuts import render

from netbox_proxbox.models import FastAPIEndpoint, ProxmoxEndpoint
from netbox_proxbox.utils import get_fastapi_url

from django.views.decorators.http import require_GET

from django.http import HttpRequest, HttpResponse
from django_htmx.middleware import HtmxDetails
class HtmxHttpRequest(HttpRequest):
    htmx: HtmxDetails

import requests

@require_GET
def get_proxmox_card(
    request: HtmxHttpRequest,
    pk: int,
) -> HttpResponse:
    template_name = 'netbox_proxbox/home/proxmox_card.html'
    
    fastapi_info = None
    fastapi_url = None
    try:
        fastapi_object = FastAPIEndpoint.objects.first()
        fastapi_info = get_fastapi_url(fastapi_object)
        if fastapi_info is not None:
            fastapi_url = fastapi_info.get('http_url')
    except FastAPIEndpoint.DoesNotExist:
        pass
    
    proxmox_object = None
    try:
        proxmox_object = ProxmoxEndpoint.objects.get(pk=pk)
    except ProxmoxEndpoint.DoesNotExist:
        pass
        
    version_data: list = []
    cluster_data: list = []
    
    if fastapi_url and fastapi_info and proxmox_object:
        domain = str(proxmox_object.ip_address).split('/')[0]
        
        version_endpoint = f'{fastapi_url}/proxmox/version?&domain={domain}'
        print('version_endpoint:', version_endpoint)
        cluster_endpoint = f'{fastapi_url}/proxmox/sessions?domain={domain}'
        
        try:
            version_response = requests.get(version_endpoint)
            cluster_response = requests.get(cluster_endpoint)
            
            version_response.raise_for_status()
            cluster_response.raise_for_status()
            
            version_data = version_response.json()
            print('version-data:', version_data)
            cluster_data = cluster_response.json()
            print('cluster-data:', cluster_data)
        except Exception as error:
            print('HTTP Error ocurred:', error)
            pass
    
    '''
    version_data example:
    [
        {
            "CLUSTER-NAME": {
                "version": "8.3.0",
                "release": "8.3",
                "repoid": "c1689ccb1065a83b"
            }
        }
    ]
    
    cluster_endpoint example:
    [
        {
            "domain": "10.0.0.1",
            "http_port": 8006,
            "user": "root@pam",
            "name": "CLUSTER-NAME",
            "mode": "cluster"
        }
    ]
    '''
    
    # Extract the version and cluster data from the JSON response
    for version, cluster in zip(version_data, cluster_data):
        print(version, cluster)
        for key, value in version.items():
            version_data = value

        cluster_data = cluster
    
    # Combine the version and cluster data into one dictionary
    print(cluster_data, version_data)
    all_data: dict = {}
    if type(cluster_data) is dict and type(version_data) is dict:
        all_data = cluster_data | version_data
    
    return render(
        request,
        template_name,
        {
            'cluster_data': all_data,
            'object': proxmox_object,
        }
    )


from packaging import version
from django.conf import settings

NETBOX_RELEASE_CURRENT = version.parse(settings.VERSION)
NETBOX_RELEASE_28 = version.parse("2.8")
NETBOX_RELEASE_29 = version.parse("2.9")
NETBOX_RELEASE_210 = version.parse("2.10")


import asyncio
import websockets
from asgiref.sync import async_to_sync, sync_to_async
from netbox_proxbox.views import get_fastapi_url
from netbox_proxbox.models import FastAPIEndpoint
from django.shortcuts import render
from django.http import HttpResponse, JsonResponse
import threading
from django.views import View
from django_htmx.http import HttpResponseClientRedirect
from queue import Queue
import json

GLOBAL_WEBSOCKET_MESSAGES = []
websocket_task = None
websocket_lock = threading.Lock()  # Add a lock to ensure thread safety
message_queue = Queue()
sync_processes = {
    'full-update': 'not-started',
    'devices': 'not-started',
    'virtual-machines': 'not-started',
}

async def websocket_client(uri):
    print('websocket_client reached')
    try:
        print(f'Connecting with websocket server: {uri}')
        async with websockets.connect(f'{uri}') as websocket:
            while True:
                # Send new messages from the queue
                if not message_queue.empty():
                    new_message = message_queue.get()
                    
                    if new_message == 'Sync Nodes':
                        print(f'Sending message: {new_message}')
                        await websocket.send(new_message)
                        sync_processes['devices'] = 'syncing'
                    
                    print(f'Sending message: {new_message}')
                    await websocket.send(new_message)

                response = await websocket.recv()
                response_dict = {}
                try:
                    # Convert the response to a dictionary
                    response_dict = json.loads(response)
                except json.JSONDecodeError:
                    pass
                
                print(f'response_type: {type(response)}')
                if response_dict:
                    print(f'response_dict: {response_dict}')
                    if all([
                        response_dict.get('object') == 'device',  # Checks if 'object' key exists and is 'device'
                        response_dict.get('end') == True,  # Ensures 'end' key exists and has a truthy value
                        sync_processes.get('devices') == 'syncing'  # Ensures 'devices' exists and is 'syncing'
                    ]):
                        sync_processes['devices'] = 'not-started'
                
                with websocket_lock:
                    GLOBAL_WEBSOCKET_MESSAGES.append(response)
                print('GLOBAL_WEBSOCKET_MESSAGES:', GLOBAL_WEBSOCKET_MESSAGES)
    except Exception as e:
        print(f'WebSocket connection error: {e}')
        await asyncio.sleep(5)  # Wait for 5 seconds before attempting to reconnect
        await websocket_client(uri)

def start_websocket(uri):
    global websocket_task
    if websocket_task is None or websocket_task.done():
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)
        websocket_task = loop.create_task(websocket_client(uri))
        threading.Thread(target=loop.run_forever).start()
        print('WebSocket task started.')

def stop_websocket():
    global websocket_task
    if websocket_task:
        websocket_task.cancel()
        print('WebSocket task stopped.')

def send_message(message):
    message_queue.put(message)

class WebSocketView(View):
    template_name = 'netbox_proxbox/websocket_page.html'
    htmx_template_name = 'netbox_proxbox/partials/websocket_messages.html'
    
    def get(self, request, message):
        print(f'sync_processes: {sync_processes}')
        print(message_queue)
        
        
        # Access `json_response` from kwargs
        json_response = request.GET.get('json_response', 'false').lower() == 'true'
        print(f'message: {message}')
        print(f'json_response: {json_response}')
        
        bulk_messages_count = 20
        # Declare the global variable to store the messages
        global GLOBAL_WEBSOCKET_MESSAGES
        global websocket_task
        
        # Ensure thread safety for message access
        with websocket_lock:
            # Get the first 20 messages
            messages_to_render = GLOBAL_WEBSOCKET_MESSAGES[:bulk_messages_count]
            
            # After saving the first 20 messages, remove them from the list
            GLOBAL_WEBSOCKET_MESSAGES = GLOBAL_WEBSOCKET_MESSAGES[bulk_messages_count:]
        
        # Use sync_to_async to call synchronous Django ORM operations
        fastapi_object = FastAPIEndpoint.objects.first()
        if fastapi_object is None:
            return HttpResponse("FastAPIEndpoint object not found", status=404)
        
        uri = get_fastapi_url(fastapi_object).get('websocket_url')
        if uri is None:
            return HttpResponse("WebSocket URL not found", status=404)
        
        # Start websocket only if not already running
        if not websocket_task or websocket_task.done():
            start_websocket(uri)
        
        # Send the initial message
        if message == 'full-update' and sync_processes.get('full-update') == 'not-started':
            sync_processes['full-update'] = 'syncing'
            send_message('Full Update')
        elif message == 'devices' and sync_processes.get('devices') == 'not-started':
            send_message('Sync Nodes')
        elif message == 'virtual-machines' and sync_processes.get('virtual-machines') == 'not-started':
            sync_processes['virtual-machines'] = 'syncing'
            send_message('Sync Virtual Machines')
        

        print('GLOBAL_WEBSOCKET_MESSAGES:', GLOBAL_WEBSOCKET_MESSAGES)
        print(websocket_task)
        
        context = {
            'messages': messages_to_render,
        }
        
        if json_response:
            # safe=False is used to allow non-dict objects to be serialized
            return JsonResponse(messages_to_render, safe=False)
        
        if request.htmx:
            # Return partial update for HTMX request
            return render(request, self.htmx_template_name, context)
        else:
            # Return full page render for non-HTMX request
            return render(request, self.template_name, context)



import re
from django.db import models
from django.core.exceptions import ValidationError

def validate_domain(value):
    domain_regex = re.compile(
        r'^(?:[a-zA-Z0-9]'  # First character of the domain
        r'(?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?\.)'  # Sub domain + hostname
        r'+[a-zA-Z]{2,6}$'  # Top level domain
    )
    if value != 'localhost':
        if not domain_regex.match(value):
            raise ValidationError(f'{value} is not a valid domain name')

class DomainField(models.CharField):
    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = 253  # Maximum length of a domain name is 253 characters
        super().__init__(*args, **kwargs)
        self.validators.append(validate_domain)

# Testing the validate_domain function separately
try:
    validate_domain("example.com")  # Valid domain
    validate_domain("sub.example.com")  # Valid domain
    validate_domain("localhost")    # Valid localhost
    validate_domain("invalid_domain") # Invalid domain
except ValidationError as e:
    print(e)


from netbox.api.viewsets import NetBoxModelViewSet

from .. import filtersets, models
from .serializers import (
    ProxmoxEndpointSerializer,
    NetBoxEndpointSerializer,
    FastAPIEndpointSerializer,
    SyncProcessSerializer,
)


class SyncProcessViewSet(NetBoxModelViewSet):
    queryset = models.SyncProcess.objects.all()
    serializer_class = SyncProcessSerializer
    filterset_class = filtersets.SyncProcessFilterSet

class ProxmoxEndpointViewSet(NetBoxModelViewSet):
    queryset = models.ProxmoxEndpoint.objects.all()
    serializer_class = ProxmoxEndpointSerializer

class NetBoxEndpointViewSet(NetBoxModelViewSet):
    queryset = models.NetBoxEndpoint.objects.all()
    serializer_class = NetBoxEndpointSerializer


class FastAPIEndpointViewSet(NetBoxModelViewSet):
    queryset = models.FastAPIEndpoint.objects.all()
    serializer_class = FastAPIEndpointSerializer

    


import requests

def get(**kwargs):
    
    owner = "netdevopsbr"
    repo = "netbox-proxbox"
    #branch = "develop"
    
    # Get variable passed from function
    if kwargs.get("filename"):
        filename = kwargs.get("filename")

        # Construct the API endpoint URL
        url = f"https://api.github.com/repos/{owner}/{repo}/contents/{filename}"
        
        # Make the GET request
        response = requests.get(url)

        # Check if the request was successful (status code 200)
        if response.status_code == 200:
            # Retrieve the content from the response
            content_url = response.json()["download_url"]
        
            if content_url:
                markdown_content = requests.get(content_url)
                
                if markdown_content.status_code == 200:
                    return markdown_content.text
        else:
            # Print the error message if the request was not successful
            print(f"Error: {response.status_code} - {response.json()['message']}")


from django.db.models import Q
from netbox.filtersets import NetBoxModelFilterSet
from .models import ProxmoxEndpoint, NetBoxEndpoint, FastAPIEndpoint, SyncProcess


class SyncProcessFilterSet(NetBoxModelFilterSet):
    """
    FilterSet for SyncProcess model.
    It is used in the SyncProcessListView.
    """
    class Meta:
        model = SyncProcess
        fields = ['id', 'name', 'sync_type', 'status', 'started_at', 'completed_at', 'runtime']

    def search(self, queryset, name, value):
        return queryset.filter(name__icontains=value)


class ProxmoxEndpointFilterSet(NetBoxModelFilterSet):
    class Meta:
        model = ProxmoxEndpoint
        fields = ['id', 'name', 'domain', 'ip_address', 'mode']
    
    def search(self, queryset, name, value):
            return queryset.filter(name__icontains=value)


class NetBoxEndpointFilterSet(NetBoxModelFilterSet):
    class Meta:
        model = NetBoxEndpoint
        fields = ['id', 'name', 'domain', 'ip_address']

    def search(self, queryset, name, value):
        return queryset.filter(name__icontains=value)


class FastAPIEndpointFilterSet(NetBoxModelFilterSet):
    class Meta:
        model = FastAPIEndpoint
        fields = ['id', 'name', 'domain', 'ip_address']
    
    def search(self, queryset, name, value):
        return queryset.filter(name__icontains=value)



# NetBox Imports
from netbox.views import generic

# Proxbox Imports
from netbox_proxbox.models import SyncProcess
from netbox_proxbox.tables import SyncProcessTable
from netbox_proxbox.filtersets import SyncProcessFilterSet
from netbox_proxbox.forms import SyncProcessForm, SyncProcessFilterForm

__all__ = (
    'SyncProcessView',
    'SyncProcessListView',
    'SyncProcessEditView',
    'SyncProcessDeleteView',
)

class SyncProcessView(generic.ObjectView):
    """
    Display a single sync process.
    """
    queryset = SyncProcess.objects.all()

class SyncProcessListView(generic.ObjectListView):
    """
    Display a list of sync processes.
    """
    queryset = SyncProcess.objects.all()
    table = SyncProcessTable
    filterset = SyncProcessFilterSet
    filterset_form = SyncProcessFilterForm

class SyncProcessEditView(generic.ObjectEditView):
    """
    Edit a sync process.
    """
    queryset = SyncProcess.objects.all()
    form = SyncProcessForm

class SyncProcessDeleteView(generic.ObjectDeleteView):
    """
    Delete a sync process.
    """
    queryset = SyncProcess.objects.all()




# Django Imports
from django import forms

# NetBox Imports
from utilities.forms.fields import DynamicModelChoiceField, CommentField
from netbox.forms import NetBoxModelForm, NetBoxModelFilterSetForm
from ipam.models import IPAddress
from users.models import Token

# Proxbox Imports
from ..models import NetBoxEndpoint


class NetBoxEndpointForm(NetBoxModelForm):
    """
    Form for NetBoxEndpoint model.
    It is used to CREATE and UPDATE NetBoxEndpoint objects.
    """
    
    ip_address = DynamicModelChoiceField(
        queryset=IPAddress.objects.all(),
        required=False,
        help_text='Select IP Address',
        label='IP Address'
    )
    
    token = forms.ModelChoiceField(
        queryset=Token.objects.all(),
        required=False,
        help_text='Choose an existing NetBox API Token',
        label='API Token',
        to_field_name='key'  # This will return the full token from the database
    )
    
    comments = CommentField()
    
    class Meta:
        model = NetBoxEndpoint
        fields = (
            'name', 'domain', 'ip_address', 'port',
            'token', 'verify_ssl', 'tags'
        )


class NetBoxEndpointFilterForm(NetBoxModelFilterSetForm):
    """
    Filter form for NetBoxEndpoint model.
    It is used in the NetBoxEndpointListView.
    """
    
    model = NetBoxEndpoint
    name = forms.CharField(
        required=False
    )
    ip_address = forms.ModelMultipleChoiceField(
        queryset=IPAddress.objects.all(),
        required=False,
        help_text='Select IP Address'
    )


import django_tables2 as tables
from django.utils.translation import gettext as _

from netbox.tables import NetBoxTable, ChoiceFieldColumn
from netbox.tables.columns import BooleanColumn

from .models import (
    ProxmoxEndpoint,
    NetBoxEndpoint,
    FastAPIEndpoint,
    SyncProcess,
)


class SyncProcessTable(NetBoxTable):
    name = tables.Column(linkify=True)
    sync_type = ChoiceFieldColumn(
        verbose_name=_('Sync Type'),
    )
    status = ChoiceFieldColumn(
        verbose_name=_('Status'),
    )
    started_at = tables.Column(
        verbose_name=_('Started At'),
    )
    completed_at = tables.Column(
        verbose_name=_('Completed At'),
    )
    last_updated = tables.Column(
        verbose_name=_('Last Updated'),
    )
    runtime = tables.Column(
        verbose_name=_('Runtime'),
    )
    
    class Meta(NetBoxTable.Meta):
        model = SyncProcess
        fields = (
            'pk', 'id', 'name', 'sync_type', 'status', 'started_at', 'completed_at', 'last_updated',
            'runtime', 'actions',
        )
    
        default_columns = (
            'pk', 'name', 'sync_type', 'status', 'started_at', 'completed_at', 'runtime'
        )
        
        order_by = ('-id',)

class ProxmoxEndpointTable(NetBoxTable):
    name = tables.Column(linkify=True)
    ip_address = tables.Column(linkify=True)
    mode = ChoiceFieldColumn()
    verify_ssl = BooleanColumn()
    
    class Meta(NetBoxTable.Meta):
        model = ProxmoxEndpoint
        fields = (
            'pk', 'id', 'name', 'domain', 'ip_address', 'port',
            'mode', 'version', 'repoid', 'username', 'token_name',
            'verify_ssl', 'actions',
        )
        
        default_columns = (
            'pk', 'name', 'domain', 'ip_address', 'port', 'mode',
            'version', 'verify_ssl'
        )


class NetBoxEndpointTable(NetBoxTable):
    name = tables.Column(linkify=True)
    ip_address = tables.Column(linkify=True)
    verify_ssl = BooleanColumn()
    
    class Meta(NetBoxTable.Meta):
        model = NetBoxEndpoint
        fields = (
            'pk', 'id', 'name', 'ip_address', 'port',
            'verify_ssl', 'token', 'actions',
        )
        
        default_columns = (
            'pk', 'name', 'ip_address', 'port', 'verify_ssl', 'token'
        )


class FastAPIEndpointTable(NetBoxTable):
    name = tables.Column(linkify=True)
    ip_address = tables.Column(linkify=True)
    verify_ssl = BooleanColumn()
    
    class Meta(NetBoxTable.Meta):
        model = FastAPIEndpoint
        fields = (
            'pk', 'id', 'name', 'domain', 'ip_address', 'port',
            'verify_ssl', 'use_websocket', 'websocket_domain', 'websocket_port', 'server_side_websocket', 'token', 'actions',
        )
        
        default_columns = (
            'pk', 'name', 'domain', 'ip_address', 'port', 'verify_ssl', 'use_websocket', 'websocket_domain'
        )


# Django Imports
from django.http import HttpRequest, HttpResponse
from django.shortcuts import render
from django.views.decorators.http import require_GET
from django.urls import reverse
import requests

# Django-HTMX Imports
from django_htmx.middleware import HtmxDetails
from django_htmx.http import replace_url

class HtmxHttpRequest(HttpRequest):
    htmx: HtmxDetails

from netbox_proxbox.models import *
from netbox_proxbox.utils import get_fastapi_url

CONNECTED_URL_SUCCESSFUL = None

def fastapi_status(pk: int) -> dict:
    global CONNECTED_URL_SUCCESSFUL
    connected: bool = False
    
    fastapi_service_obj = None
    try:
        fastapi_service_obj = FastAPIEndpoint.objects.get(pk=pk)
    except FastAPIEndpoint.DoesNotExist:
        fastapi_service_obj = FastAPIEndpoint.objects.first()
        
    if fastapi_service_obj:
        fastapi_detail = get_fastapi_url(fastapi_service_obj)
        fastapi_url: str = fastapi_detail.get('http_url')
        fastapi_verify_ssl: bool = fastapi_detail.get('verify_ssl', True)

        print(f'FastAPI URL: {fastapi_url}')
        
        if fastapi_url:
            try:
                response = requests.get(fastapi_url, verify=fastapi_verify_ssl)
                print(f'FastAPI response: {response.json()}')
                response.raise_for_status()
                connected = True
                CONNECTED_URL_SUCCESSFUL = fastapi_url
            except Exception as errr:
                print(f'Error ocurred: {errr}')
                
                # Try to connect to FastAPI using the IP address and port.
                print(f'Trying to connect to FastAPI using the IP address and port: {fastapi_detail.get("ip_address_url")}')
                response = requests.get(fastapi_detail.get('ip_address_url'), verify=False)
                print(f'FastAPI response: {response.json()}')
                connected = True
                CONNECTED_URL_SUCCESSFUL = fastapi_detail.get('ip_address_url')
            '''
            except requests.exceptions.HTTPError as err:
                print(f'HTTP error ocurred: {err}')
                response = requests.get(fastapi_detail.get('ip_address_url'), verify=False)
                print(f'FastAPI response: {response.json()}')
                response.raise_for_status()
                connected = True
            '''
            
    return {
        'url': fastapi_url,
        'connected': connected
    }
    
def netbox_status(pk: int, base_url: str) -> str:
    netbox_service_obj = None
    status = 'error'
    
    try:
        # Get NetBoxEndpoint object by primary key.
        netbox_service_obj = NetBoxEndpoint.objects.get(pk=pk)
    except NetBoxEndpoint.DoesNotExist:
        return status
    
    # Get NetBoxEndpoint IP address.
    default_netbox_ip = "127.0.0.1"
    netbox_ip = default_netbox_ip
    try:
        netbox_ipaddress_obj = getattr(netbox_service_obj,  "ip_address", None)
        if netbox_ipaddress_obj:
            netbox_ip = getattr(netbox_ipaddress_obj, "address")
            netbox_ip = netbox_ip.split('/')[0] if netbox_ip else default_netbox_ip
    except Exception as err:
        print(f"Error getting NetBoxEndpoint IP address: {err}. Using default IP address: {default_netbox_ip}")
        netbox_ip = default_netbox_ip
    
    # Define NetBoxEndpoint URL to get endpoints from pynetbox-api database (sqlite)
    netbox_endpoint_url: str = f'{base_url}/netbox/endpoint'
    netbox_status_route: str = f'{base_url}/netbox/status'
    
    try:
        # Check if NetBoxEndpoint exists on FastAPI database.
        response = requests.get(netbox_endpoint_url)
        response.raise_for_status()
        response = list(response.json())
        
        netbox = None
        
        token_obj = getattr(netbox_service_obj, 'token', None)
        token_key = getattr(token_obj, 'key', None) if token_obj else 'invalid-token'
        
        current_netbox: dict = {
            'id': pk,
            'name': netbox_service_obj.name if netbox_service_obj.name else None,
            'ip_address': netbox_ip,
            'domain': netbox_service_obj.domain if netbox_service_obj.domain else None,
            'port': netbox_service_obj.port if netbox_service_obj.port else None,
            'token': token_key,
            'verify_ssl': netbox_service_obj.verify_ssl if netbox_service_obj.verify_ssl else False,
        }
        
        # If no NetBoxEndpoints are found on FastAPI database, create a new one.
        if len(response) == 0:
            print('No NetBoxEndpoints found on FastAPI database.')
            
            # Create NetBoxEndpoint on FastAPI database.
            print('Creating NetBoxEndpoint on FastAPI database...')
            print(f'current_netbox: {current_netbox}')
            print(f'netbox_endpoint_url: {netbox_endpoint_url}')
            requests.post(netbox_endpoint_url, json=current_netbox)
        
        if len(response) > 0:
            # Delete all NetBoxEndpoints from FastAPI database, except the one that matches the current NetBoxEndpoint.
            for netbox_endpoint in response:
                if netbox_endpoint['id'] != pk:
                    requests.delete(f'{netbox_endpoint_url}/{netbox_endpoint["id"]}')
                else:
                    # If NetBox ID exists, update it (if needed)
                    if netbox_endpoint != current_netbox:
                        updated_netbox_endpoint: dict = netbox_endpoint | current_netbox
                        requests.delete(f'{netbox_endpoint_url}/{netbox_endpoint["id"]}') 
                        requests.post(netbox_endpoint_url, json=updated_netbox_endpoint)
            
        else:
            # Create NetBoxEndpoint on FastAPI database.
            print('Creating NetBoxEndpoint on FastAPI database...')
            requests.post(netbox_endpoint_url, json=current_netbox)
        
        # NetBoxEndpoint exists on FastAPI database. Check if it is alive.
        try:
            response = requests.get(netbox_status_route)
            response.raise_for_status()
            status = 'success'
        except requests.exceptions.HTTPError as err:
            print(f'HTTP error ocurred: {err}')
            status = 'error'
        except Exception as errr:
            print(f'Error ocurred: {errr}')
            status = 'error'
        
    except requests.exceptions.HTTPError as err:
        print(f'HTTP error ocurred: {err}')
        status = 'error'
    
    except Exception as errr:
        print(f'Error ocurred: {errr}')
        status = 'error'
    
    return status
    
def proxmox_status(pk: int, base_url: str) -> str:
    proxmox_service_obj = None
    status = 'error'
    
    try:
        proxmox_service_obj = ProxmoxEndpoint.objects.get(pk=pk)    
    except ProxmoxEndpoint.DoesNotExist:
        return status
        
    if proxmox_service_obj:
        proxmox_ip_address: str = str(proxmox_service_obj.ip_address).split('/')[0]
        proxmox_domain: str = proxmox_service_obj.domain if proxmox_service_obj.domain else None
        
        url = None
        if proxmox_domain:
            url = f'{base_url}/proxmox/version?domain={proxmox_domain}'
        
        if not proxmox_domain and proxmox_ip_address:
            url = f'{base_url}/proxmox/version?ip_address={proxmox_ip_address}'
        
        
        if url:
            try:
                response = requests.get(url)
                print(response)
                print(response.status_code, type(response.status_code))
                response.raise_for_status()
                status = 'success'
            except requests.exceptions.HTTPError as err:
                print(f'HTTP error ocurred: {err}')
            except Exception as errr:
                print(f'Error ocurred: {errr}')
    
    print(status)
    return status

@require_GET
def get_service_status(
    request: HtmxHttpRequest,
    service: str,
    pk: int,
) -> HttpResponse:
    """Get the status of a service."""
    template_name: str = 'netbox_proxbox/status_badge.html'
    
    # Accept only HTMX requests to render this view.
    #if not request.htmx:
    #    return HttpResponse(status=400)

    status: str = 'unknown'
    fastapi_response: dict = {}
    global CONNECTED_URL_SUCCESSFUL
    
    if service == 'fastapi':
        fastapi_response = fastapi_status(pk)
        status = 'success' if fastapi_response.get('connected') == True else 'error'
    else:
        try:
            fastapi_response = fastapi_status(pk=FastAPIEndpoint.objects.first().id)
        except FastAPIEndpoint.DoesNotExist:
            pass
    
    if service == 'netbox' and fastapi_response.get('connected') == True:
        print('Trying to get NetBox status...')
        netbox_response = netbox_status(pk=pk, base_url=CONNECTED_URL_SUCCESSFUL)
        status = netbox_response if netbox_response is not None else 'error'

    if service == 'proxmox' and fastapi_response.get('connected') == True:
        print('Trying to get Proxmox status...')
        proxmox_response = proxmox_status(pk=pk, base_url=CONNECTED_URL_SUCCESSFUL)
        status = proxmox_response if proxmox_response is not None else 'error'
    
    return render(
        request,
        template_name,
        {
            'status': status
        }
    )
 


# Django Imports
from django import forms
from django.utils.translation import gettext as _
# NetBox Imports
from utilities.forms.fields import DynamicModelChoiceField, CommentField
from netbox.forms import NetBoxModelForm, NetBoxModelFilterSetForm

# Proxbox Imports
from ..models import SyncProcess
from ..choices import SyncTypeChoices, SyncStatusChoices

class SyncProcessForm(NetBoxModelForm):
    """
    Form for SyncProcess model.
    It is used to CREATE and UPDATE SyncProcess objects.
    """
    
    comments = CommentField()
    
    sync_type = forms.ChoiceField(
        choices=SyncTypeChoices,
        required=False
    )
    status = forms.ChoiceField(
        choices=SyncStatusChoices,
        required=False
    )
    runtime = forms.FloatField(
        required=False,
        help_text=_('Time elapsed for the sync process. Format: seconds')
    )
    
    class Meta:
        model = SyncProcess
        fields = ('name', 'sync_type', 'status', 'started_at', 'completed_at', 'runtime')


class SyncProcessFilterForm(NetBoxModelFilterSetForm):
    """
    Filter form for SyncProcess model.
    It is used in the SyncProcessListView.
    """
    
    model = SyncProcess
    name = forms.CharField(
        required=False
    )
    sync_type = forms.ChoiceField(
        choices=SyncTypeChoices,
        required=False,
    )
    status = forms.ChoiceField(
        choices=SyncStatusChoices,
        required=False,
    )
    runtime = forms.FloatField(
        required=False,
        help_text=_('Time elapsed for the sync process. Format: seconds')
    )



# Generated by Django 5.1.3 on 2025-04-01 19:45

import django.core.validators
import django.db.models.deletion
import netbox_proxbox.fields
import taggit.managers
import utilities.json
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('extras', '0122_charfield_null_choices'),
        ('ipam', '0076_natural_ordering'),
        ('netbox_proxbox', '0007_alter_fastapiendpoint_name'),
        ('users', '0009_update_group_perms'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='proxmoxendpoint',
            unique_together=set(),
        ),
        migrations.AddField(
            model_name='fastapiendpoint',
            name='domain',
            field=netbox_proxbox.fields.DomainField(default='localhost', max_length=253),
        ),
        migrations.AddField(
            model_name='fastapiendpoint',
            name='server_side_websocket',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='fastapiendpoint',
            name='token',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='fastapiendpoint',
            name='use_websocket',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='fastapiendpoint',
            name='websocket_domain',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='fastapiendpoint',
            name='websocket_port',
            field=models.PositiveIntegerField(default=8800, validators=[django.core.validators.MinValueValidator(1), django.core.validators.MaxValueValidator(65535)]),
        ),
        migrations.AddField(
            model_name='netboxendpoint',
            name='domain',
            field=netbox_proxbox.fields.DomainField(default='localhost', max_length=253),
        ),
        migrations.AddField(
            model_name='proxmoxendpoint',
            name='domain',
            field=netbox_proxbox.fields.DomainField(blank=True, max_length=253, null=True),
        ),
        migrations.AlterField(
            model_name='fastapiendpoint',
            name='ip_address',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='ipam.ipaddress'),
        ),
        migrations.AlterField(
            model_name='netboxendpoint',
            name='ip_address',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='ipam.ipaddress'),
        ),
        migrations.AlterField(
            model_name='netboxendpoint',
            name='token',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='users.token'),
        ),
        migrations.AlterField(
            model_name='proxmoxendpoint',
            name='ip_address',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='ipam.ipaddress'),
        ),
        migrations.AlterField(
            model_name='proxmoxendpoint',
            name='verify_ssl',
            field=models.BooleanField(default=False),
        ),
        migrations.AlterUniqueTogether(
            name='proxmoxendpoint',
            unique_together={('name', 'ip_address', 'domain')},
        ),
        migrations.CreateModel(
            name='SyncProcess',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('name', models.CharField(max_length=255, unique=True)),
                ('sync_type', models.CharField(default='all', max_length=20)),
                ('status', models.CharField(default='not-started', max_length=20)),
                ('started_at', models.DateTimeField(blank=True, null=True)),
                ('completed_at', models.DateTimeField(blank=True, null=True)),
                ('runtime', models.FloatField(blank=True, null=True)),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# Django Imports
from django.shortcuts import get_object_or_404
from django.urls import reverse

# NetBox Imports
from netbox.views import generic

# ProxBox Imports
from netbox_proxbox.models import NetBoxEndpoint
from netbox_proxbox.tables import NetBoxEndpointTable
from netbox_proxbox.filtersets import NetBoxEndpointFilterSet
from netbox_proxbox.forms import NetBoxEndpointForm, NetBoxEndpointFilterForm


__all__ = (
    'NetBoxEndpointView',
    'NetBoxEndpointListView',
    'NetBoxEndpointEditView',
    'NetBoxEndpointDeleteView',
)


class NetBoxEndpointView(generic.ObjectView):
    queryset = NetBoxEndpoint.objects.all()


class NetBoxEndpointListView(generic.ObjectListView):
    queryset = NetBoxEndpoint.objects.all()
    table = NetBoxEndpointTable
    filterset = NetBoxEndpointFilterSet
    filterset_form = NetBoxEndpointFilterForm


class NetBoxEndpointEditView(generic.ObjectEditView):
    """
    This view is used to edit and create the NetBoxEndpoint object.
    
    If there is already an existing NetBoxEndpoint object,
    the view will return the existing object, allowing only one object to be created.
    """
    
    template_name = 'netbox_proxbox/netboxendpoint_edit.html'
    queryset = NetBoxEndpoint.objects.all()
    form = NetBoxEndpointForm
    
    def get_object(self, **kwargs):
        # If there is already an existing NetBoxEndpoint object, return the first object
        if int(NetBoxEndpoint.objects.count()) >= 1:
            return NetBoxEndpoint.objects.first()
        
        if not kwargs:
            # We're creating a new object
            return self.queryset.model()
            
        # If there is no existing NetBoxEndpoint object, return the object with the given kwargs
        return get_object_or_404(NetBoxEndpoint.objects.all(), **kwargs)
    
    def get_extra_context(self, request, instance):
        # If there is already an existing NetBoxEndpoint object, pass True to the template
        if int(NetBoxEndpoint.objects.count()) >= 1:
            return {'existing_object': True}
        
        # If there is no existing NetBoxEndpoint object, pass False to the template
        return {'existing_object': False}


class NetBoxEndpointDeleteView(generic.ObjectDeleteView):
    queryset = NetBoxEndpoint.objects.all()

    


from netbox.plugins import PluginMenuButton, PluginMenuItem, PluginMenu
#from utilities.choices import ButtonColorChoices

fullupdate_item = PluginMenuItem(
    link='plugins:netbox_proxbox:home',
    link_text='Full Update',
)

nodes_item = PluginMenuItem(
    link='plugins:netbox_proxbox:nodes',
    link_text='Nodes (Devices)',
)

virtual_machines_item = PluginMenuItem(
    link='plugins:netbox_proxbox:virtual_machines',
    link_text='Virtual Machines',
)

sync_processes_item = PluginMenuItem(
    link='plugins:netbox_proxbox:syncprocess_list',
    link_text='Sync Processes',
)

contributing_item = PluginMenuItem(
    link='plugins:netbox_proxbox:contributing',
    link_text='Contributing!',
)

"""
    Endpoints Navigation Buttons.
"""
proxmox_endpoints_item = PluginMenuItem(
    link='plugins:netbox_proxbox:proxmoxendpoint_list',
    link_text='Proxmox API',
    buttons=(
        PluginMenuButton('plugins:netbox_proxbox:proxmoxendpoint_add', 'Add Proxmox Endpoint', 'mdi mdi-plus'),
    )
)

netbox_endpoints_item = PluginMenuItem(
    link='plugins:netbox_proxbox:netboxendpoint_list',
    link_text='NetBox API',
    buttons=(
        PluginMenuButton('plugins:netbox_proxbox:netboxendpoint_add', 'Add NetBox Endpoint', 'mdi mdi-plus'),
    )
)

fastapi_endpoints_item = PluginMenuItem(
    link='plugins:netbox_proxbox:fastapiendpoint_list',
    link_text='ProxBox API (FastAPI)',
    buttons=(
        PluginMenuButton('plugins:netbox_proxbox:fastapiendpoint_add', 'Add Proxbox API Endpoint', 'mdi mdi-plus'),
    )
)

community_item = PluginMenuItem(
    link='plugins:netbox_proxbox:community',
    link_text='Community',
    buttons=[
        PluginMenuButton(
            "plugins:netbox_proxbox:discussions",
            "GitHub Discussions",
            "mdi mdi-github",
            #ButtonColorChoices.GRAY,
        ),
        PluginMenuButton(
            "plugins:netbox_proxbox:discord",
            "Discord Community",
            "mdi mdi-forum",
            #ButtonColorChoices.BLACK,
        ),
        PluginMenuButton(
            "plugins:netbox_proxbox:telegram",
            "Telegram Community",
            "mdi mdi-send",
            #ButtonColorChoices.BLUE,
        ),
    ]
)


menu = PluginMenu(
    label='Proxbox',
    groups=(
        ('Proxmox Plugin', (
                fullupdate_item,
                nodes_item,
                virtual_machines_item,
                sync_processes_item,
            )
         ),
        ('Endpoints', (proxmox_endpoints_item, netbox_endpoints_item, fastapi_endpoints_item,)),
        ('Join our community', (contributing_item, community_item,)),
    ),
    icon_class='mdi mdi-dns'
)


import subprocess
import json

from django.shortcuts import render
from django.views import View

try:
    from netbox import configuration
except Exception as error:
    print(error)
    
from netbox_proxbox import ProxboxConfig
from netbox_proxbox import github

# Import other proxbox views
from .external_pages import *
from .proxbox_backend import *
from .endpoints import  *
from .keepalive_status import *
from .cards import *
from .sync_process import *
from .sync import *

from netbox_proxbox.models import ProxmoxEndpoint, NetBoxEndpoint, FastAPIEndpoint

from netbox_proxbox.utils import get_fastapi_url
    
class HomeView(View):
    """
    ## HomeView class-based view to handle incoming GET HTTP requests.
    
    ### Attributes:
    - **template_name (str):** The path to the HTML template used for rendering the view.
    
    ### Methods:
    - **get(request):**
            Handles GET requests to the view.
            Retrieves plugin configuration and default settings.
            Constructs the FastAPI endpoint URL.
            Renders the template with the configuration and default settings.
            
            **Args:**
            - **request (HttpRequest):** The HTTP request object.
            
            **Returns:**
            - **HttpResponse:** The rendered HTML response.
    """
    
    template_name = 'netbox_proxbox/home.html'

    # service incoming GET HTTP requests
    def get(self, request):
        """Get request."""
        
        default_config = dict =  getattr(ProxboxConfig, 'default_settings', {})
        
        fastapi_example_url: str = 'https://example.fastapi.com',
        fastapi_exampel_websocket_url: str = 'wss://example.fastapi.com'
                
                
        proxmox_endpoint_obj = ProxmoxEndpoint.objects.all()
        proxmox_endpoint_count = proxmox_endpoint_obj.count()
        if proxmox_endpoint_count <= 0:
            proxmox_endpoint_obj = None
        
            
        netbox_endpoint_obj = NetBoxEndpoint.objects.all()
        netbox_endpoint_count = netbox_endpoint_obj.count()
        if netbox_endpoint_count <= 0:
            netbox_endpoint_obj = None
        
        fastapi_endpoint_obj = FastAPIEndpoint.objects.all()
        fastapi_endpoint_count = fastapi_endpoint_obj.count()
        if fastapi_endpoint_count <= 0:
            fastapi_endpoint_obj = None

        fastapi_info = {}
        if fastapi_endpoint_obj is not None:
            # Get first object from FastAPIEndpoint queryset.
            fastapi_object = fastapi_endpoint_obj[0]
            fastapi_info = get_fastapi_url(fastapi_object)
        
        return render(
            request,
            self.template_name,
            {
                "default_config": default_config,
                'proxmox_endpoint_list': proxmox_endpoint_obj,
                'netbox_endpoint_list': netbox_endpoint_obj,
                'fastapi_endpoint_list': fastapi_endpoint_obj,
                'fastapi_url': fastapi_info.get('http_url', fastapi_example_url),
                'fastapi_websocket_url': fastapi_info.get('websocket_url', fastapi_exampel_websocket_url)
            }
        )

class TestWebSocketView(View):
    template_name = 'netbox_proxbox/test/websocket.html'
    
    def get(self, request):
        fastapi_endpoint_obj = FastAPIEndpoint.objects.all()

        fastapi_object = fastapi_endpoint_obj[0]
        fastapi_ip = str(fastapi_object.ip_address).split('/')[0]
        
        # Define HTTP(S) URL for FastAPI
        fastapi_url_https = f"https://{fastapi_ip}:{fastapi_object.port}"
        fastapi_url_http = f"http://{fastapi_ip}:{fastapi_object.port}"
        fastapi_url = fastapi_url_https if fastapi_object.verify_ssl else fastapi_url_http
        
        # Define (Secure) WebSocket URL for FastAPI
        fastapi_wss_url = f"wss://{fastapi_ip}:{fastapi_object.port}"
        fastapi_ws_url = f"ws://{fastapi_ip}:{fastapi_object.port}"
        fastapi_websocket_url = fastapi_wss_url if fastapi_object.verify_ssl else fastapi_ws_url
        
        return render(
            request,
            self.template_name,
            {
                'fastapi_url': fastapi_url,
                'fastapi_websocket_url': fastapi_websocket_url
            }
        )
    


class NodesView(View):
    template = 'netbox_proxbox/devices.html'

    def get(self, request):
        plugin_configuration: dict = getattr(configuration, "PLUGINS_CONFIG", {})

        return render(
            request, 
            self.template,
            {
                "configuration": plugin_configuration
            }
        )


class VirtualMachinesView(View):
    template = 'netbox_proxbox/virtual_machines.html'

    def get(self, request):
        plugin_configuration: dict = getattr(configuration, "PLUGINS_CONFIG", {})

        return render(
            request, 
            self.template,
            {
                "configuration": plugin_configuration
            }
        )

class ContributingView(View):
    """
    **ContributingView** handles the rendering of the contributing page for the Proxbox project.
    
    **Attributes:**
    - **template_name (str):** The path to the HTML template for the contributing page.
    
    **Methods:**
    - **get(request):** Handles GET HTTP requests and renders the contributing page with the content
    of the 'CONTRIBUTING.md' file and a title.
    """
    
    template_name = 'netbox_proxbox/contributing.html'

    # service incoming GET HTTP requests
    def get(self, request):
        """Get request."""

        title = "Contributing to Proxbox Project"
        
        return render(
            request,
            self.template_name,
            {
                "html": github.get(filename = "CONTRIBUTING.md"),
                "title": title,
            }
        )


class CommunityView(View):
    """
    CommunityView handles the rendering of the community page.
    
    **Attributes:**
    - **template_name (str):** The path to the HTML template for the community page.
    
    **Methods:**
    - **get(request):** Handles GET HTTP requests and renders the community page with a title.
    """
    
    
    template_name = 'netbox_proxbox/community.html'

    # service incoming GET HTTP requests
    def get(self, request):
        """Get request."""

        title = "Join our Community!"
        
        return render(
            request,
            self.template_name,
            {
                "title": title,
            }
        )


def get_fastapi_url(object):
    domain_or_ip = None
    ip = str(object.ip_address).split('/')[0]
    if object.domain:
        domain_or_ip = object.domain
    else:
        domain_or_ip = ip
    
    # Define WebSocket Domain
    websocket_domain = None
    if object.use_websocket and object.websocket_domain:
        websocket_domain = object.websocket_domain
    else:
        websocket_domain = ip
    
    # Define HTTP(S) URL for FastAPI
    fastapi_url_https = f"https://{domain_or_ip}:{object.port}"
    fastapi_url_http = f"http://{domain_or_ip}:{object.port}"
    fastapi_url = fastapi_url_https if object.verify_ssl else fastapi_url_http
    
    # Define (Secure) WebSocket URL for FastAPI
    fastapi_wss_url = f"wss://{websocket_domain}:{object.websocket_port}/ws"
    fastapi_ws_url = f"ws://{websocket_domain}:{object.websocket_port}/ws"
    fastapi_websocket_url = fastapi_wss_url if object.verify_ssl else fastapi_ws_url
    
    if any(host in fastapi_url for host in ['proxbox.backend.local', 'localhost', '127.0.0.1']):
        # If proxbox.backend.local is in the URL, set the REQUESTS_CA_BUNDLE environment variable.
        # It means user used mkcert to generate a certificate for the domain.
        # This is necessary to avoid SSL errors, so that python requests library can trust the certificate.
        
        import os
        import subprocess
        
        try:
            # Run mkcert -CAROOT to get the root certificate path.
            ca_root_folder = subprocess.run(['mkcert', '-CAROOT'], capture_output=True, text=True, check=True).stdout.strip()
            
            os.environ['REQUESTS_CA_BUNDLE'] = f'/{ca_root_folder}/rootCA.pem'
        except subprocess.CalledProcessError as e:
            print(f'Error running mkcert -CAROOT: {e}')
        
        except Exception as e:
            print(f'Error setting REQUESTS_CA_BUNDLE: {e}\nLikely because the user did not use mkcert to generate a certificate.')
    
    return {
        'domain': object.domain,
        'ip_address': object.ip_address,
        'ip_address_url': f'https://{ip}:{object.port}',
        'http_url': fastapi_url,
        'websocket_url': fastapi_websocket_url,
        'verify_ssl': object.verify_ssl
    }



from netbox.api.routers import NetBoxRouter
from . import views

app_name = 'proxbox'

router = NetBoxRouter()
router.register('endpoints/proxmox', views.ProxmoxEndpointViewSet)
router.register('endpoints/netbox', views.NetBoxEndpointViewSet)
router.register('endpoints/fastapi', views.FastAPIEndpointViewSet)
router.register('sync-processes', views.SyncProcessViewSet)

urlpatterns = router.urls


from django.views import View
from django.shortcuts import render, redirect

try:
    from netbox import configuration
except Exception as error:
    print(error)
    
def returnSudoUser():
    """
    Retrieves the sudo user and password from the plugin configuration.
    This function accesses the plugin configuration to fetch the sudo user and password
    required for FastAPI operations. If the configuration keys are not found, it catches
    the exception and prints the error.
    
    **Returns:**
    - **dict:** A dictionary containing the sudo user and password with keys "user" and "password".
    """
    
    
    plugin_configuration: dict = getattr(configuration, "PLUGINS_CONFIG", {})
    
    sudo_user: str = ""
    sudo_password: str = ""
    try:
        sudo_user = plugin_configuration["netbox_proxbox"]["fastapi"]["sudo"]["user"]
        sudo_password = plugin_configuration["netbox_proxbox"]["fastapi"]["sudo"]["password"]
        
    except Exception as error:
        print(error)
        
    return { "user": sudo_user, "password": sudo_password}


def run_command(sudo_command):
    """
    ### Executes a given sudo command using the credentials retrieved from the configuration.
    
    **Args:**
    - **sudo_command (str):** The sudo command to be executed.
    
    **Returns:**
        None
        
    **Raises:**
    - **Exception:** If there is an error retrieving the sudo user credentials or executing the command.
    
    The function performs the following steps:
    1. Retrieves the sudo user credentials (username and password) from the configuration.
    2. Executes the given sudo command, passing the password to stdin.
    3. Captures and prints the stdout and stderr of the command execution.
    4. Prints a success message if the command is executed successfully, otherwise prints the error message.
    """
    
    user: dict = {}
    #username: str = ""
    password: str = ""
    
    try:
        user = returnSudoUser()
        # username = user["user"] # IMPLEMENTATION LEFT.
        password = user["password"]
    except Exception as error:
        print(f"Not able to get sudo user and password from 'configuration.py'\n{error}")
    
    try:
        # Run the command and pass the password to stdin
        result = subprocess.run(
            sudo_command, 
            input=password + '\n',   # Pass the password
            stdout=subprocess.PIPE,  # Capture stdout
            stderr=subprocess.PIPE,  # Capture stderr
            text=True                # Use text mode for strings
        )  
        
        # Check the result
        if result.returncode == 0:
            print(f"Command '{sudo_command}' correctly issued.")
            return result.stdout
        else:
            print(f"Failed to run Command '{sudo_command}' Error:", result.stderr)
            return None
    
    except Exception as e:
        print(f"An error occurred: {e}")
        return None


def find_proxbox_service_in_ls(service_name: str, folder_path: str):
    """
    ### Find the Proxbox service in the list of services.
    
    This function uses the `ls` command to list the services in the system and
    searches for the Proxbox service in the output.
    
    **Returns:**
    - **bool:** True if the Proxbox service is found, False otherwise.
    """
    
    output:str = str(run_command(['sudo', '-S', 'ls', '-l', folder_path]))
    
    for line in output.splitlines():
        if service_name in line:
            return True

    return False


def check_proxbox_service_file():
    """
    Checks for the existence of the `proxbox.service`' file in the systemd folder.
    If the file does not exist, attempts to download it from a specified GitHub repository.
   
    **Steps:**
    1. Checks if the `proxbox.service` file exists in the '/etc/systemd/system' directory.
    2. If the file is not found, changes the directory to '/etc/systemd/system'.
    3. Attempts to download the `proxbox.service` file from the GitHub repository.
    4. Rechecks if the `proxbox.service` file exists after the download attempt.
    
    **Returns:**
    - **bool:** True if the `proxbox.service` file is found or successfully downloaded, False otherwise.
    """
    
    branch: str = 'develop'
    systemd_folder: str = '/etc/systemd/system'
    service_name: str = 'proxbox.service'
    github_url: str = f'https://raw.githubusercontent.com/netdevopsbr/netbox-proxbox/{branch}/contrib/{service_name}'
    
    file_exists: bool = False
    
    file_exists = find_proxbox_service_in_ls(service_name=service_name, folder_path=systemd_folder)
        
    if not file_exists:
        print("Proxbox service file not found.")
        
        try:
            run_command(['sudo', '-S', 'wget', '-P', systemd_folder, github_url])
            run_command(['sudo', '-S', 'systemctl', 'daemon-reload'])
            
        except Exception as error:
            print(f"Error getting proxbox.service file.\n   > {error}")
            return False

        file_exists = find_proxbox_service_in_ls(service_name=service_name, folder_path=systemd_folder)
        
        if file_exists:
            print("Proxbox service file found.")
            return True
    else:
        print("Proxbox service file found.")
        return True
    
    print(f"[ERROR] Proxbox service file not found and not able to get it from GitHub.\nURL to Download it: {github_url}")
    return False


def change_proxbox_service(desired_state: str):
    """
    ### Change the state of the Proxbox service.
    
    This function attempts to start, restart, or stop the Proxbox service
    based on the provided desired state. It uses system commands to manage
    the service.
    
    **Args:**
    - **desired_state (str):** The desired state of the Proxbox service. 
    It can be "start", "restart", or "stop".
    
    **Raises:**
    - **Exception:** If an error occurs while attempting to change the Proxbox service state.
    """
    
    if check_proxbox_service_file():
        print(f"Proxbox service file found. Try to change Service State to: {desired_state}")
        try:
            if desired_state == "start": 
                print("START PROXBOX")
                run_command(['sudo', '-S', 'systemctl', 'start', 'proxbox'])
                
            if desired_state == "restart":
                print("RESTART PROXBOX")
                run_command(['sudo', '-S', 'systemctl', 'restart', 'proxbox'])     
                
            if desired_state == "stop":
                print("STOP PROXBOX")
                run_command(['sudo', '-S', 'systemctl', 'stop', 'proxbox'])
            
        except Exception as error:
            print(f"Error occured trying to change Proxbox status.\n   > {error}")
    else:
        print("Proxbox service file not found. Not able to change Service State.")
        return False

        
class FixProxboxBackendView(View):
    """
    ### View to handle the fixing of the Proxbox backend service.
    
    **Attributes:**
    - **template_name (str):** The path to the HTML template for this view.
    
    **Methods:**
    - **get(request):** Handles GET requests to start the Proxbox service and redirect to the home page.
    If an error occurs while starting the service, it prints the error.
    """

    template_name = 'netbox_proxbox/fix-proxbox-backend.html'
    
    def get(self, request):
        try:
            check_proxbox_service_file()
            
            change_proxbox_service(desired_state="start")
           
        except Exception as error:
            print(error)

        return redirect('plugins:netbox_proxbox:home')


class StopProxboxBackendView(View):
    """
    ### StopProxboxBackendView handles the stopping of the Proxbox backend service.
    
    **Methods:**
    - **get(request):** Handles GET requests to stop the Proxbox service. Redirects to the home page of the netbox_proxbox plugin.
    - request: The HTTP request object.
    
    **Raises:**
    - **Exception:** If an error occurs while attempting to stop the Proxbox service.
    """
    
    def get(self, request):
        try:
           change_proxbox_service(desired_state="stop")
            
        except Exception as error:
            print(error)
            
        return redirect('plugins:netbox_proxbox:home')

        
class RestartProxboxBackendView(View):
    """
    ### RestartProxboxBackendView is a Django view that handles the restart of the Proxbox backend service.
   
    **Methods:**
    - **get(request):** Handles GET requests to restart the Proxbox service. It calls the change_proxbox_service function
    with the desired state set to "restart". If an exception occurs, it prints the error and redirects
    to the home page of the netbox_proxbox plugin.
    """
    
    def get(self, request):
        try:
           change_proxbox_service(desired_state="restart")
            
        except Exception as error:
            print(error)
            
        return redirect('plugins:netbox_proxbox:home')


class StatusProxboxBackendView(View):
    """
    ### A Django view to display the status of the Proxbox backend service.
    
    **Attributes:**
    - **template_name (str):** The template to render the status page.
    
    **Methods:**
    - **get(request):** Handles GET requests to retrieve and display the status of the Proxbox service.
    Executes a system command to get the status of the Proxbox service using `systemctl`.
    Parses the output and formats it for display in the template.
    Handles exceptions and errors that may occur during the command execution.
    Returns the rendered template with the status information.
    """
    
    
    template_name = "netbox_proxbox/proxbox-backend-status.html"
    
    def get(self, request):
            
        output: list = []
        status_proxbox_process: str = ""
        
        try:
            print("CONSOLE STATUS")
            status_proxbox_process = subprocess.check_output(
                ['sudo','systemctl','status','proxbox'],
                stderr=subprocess.STDOUT,
                text=True
            )
            
            print("\n\nstatus_proxbox_process", status_proxbox_process )
            output: list = status_proxbox_process.splitlines()
            
        except subprocess.CalledProcessError as e:
            # Handle the case where the command fails
            print(f"Command failed with return code {e.returncode}")
            print("Output (STDOUT + STDERR):", e.output)
            
            output: list = str(e.output).splitlines()
                        
        except Exception as error:
            print(error)
        
        if output and (len(output) > 0):
            output[0] = f"<h2>{output[0]}</h2>"
            
            loaded, loaded_value = str(output[1]).split("Loaded: ")
            output[1] = f"<strong>Loaded: <span class='badge text-bg-grey'>{loaded_value}</span></strong>"
            
            active, active_status = str(output[2]).split(": ")
            
            if "active" in active_status or "running" in active_status:
                output[2] = f"<strong>{active} Status: <span class='badge text-bg-green'>{active_status}</span>"
            if "activating" in active_status:
                output[2] = f"<strong>{active} Status: <span class='badge text-bg-yellow'>{active_status}</span>"
            if "dead" in active_status:
                output[2] = f"<strong>{active} Status: <span class='badge text-bg-red'>{active_status}</span>"
            
            docs, docs_link = str(output[3]).split(": ")
            
            output[3] = f"<strong>{docs}: <a target='_blank' href='{docs_link}'>{docs_link}</a></strong>"

            if "Main PID" in output[4]:
                main_pid, main_pid_value = str(output[4]).split(": ")
                output[4] = f"<strong>{main_pid}: <span class='badge text-bg-grey'>{main_pid_value}</span></strong>"
            
            if "Tasks" in output[5]:
                tasks, tasks_value = str(output[5]).split("Tasks: ")
                output[5] = f"Tasks: <span class='badge text-bg-grey'>{tasks_value}</span>"
                
            if "Memory" in output[6]:
                memory, memory_value = str(output[6]).split("Memory: ")
                output[6] = f"Memory: <span class='badge text-bg-grey'>{memory_value}</span>"
            
            if "CPU" in output[7]:
                cpu, cpu_value = str(output[7]).split("CPU: ")
                output[7] = f"Memory: <span class='badge text-bg-grey'>{cpu_value}</span>"
                
            if "CGroup" in output[8]:
                cgroup, cgroup_value = str(output[8]).split("CGroup: ")
                output[8] = f"CGroup: <span class='badge text-bg-grey'>{cgroup_value}</span>"
            
            
        return render(
            request,
            self.template_name,
            {
                "message": output
            }
        )


import requests

# Django Imports
from django.http import HttpRequest, HttpResponse
from django.views import View
from django.views.decorators.http import require_GET
from django.shortcuts import render

# Django-HTMX Imports
from django_htmx.middleware import HtmxDetails

# Proxbox Imports
from netbox_proxbox.utils import get_fastapi_url
from netbox_proxbox.models import FastAPIEndpoint

from threading import Thread


class HtmxHttpRequest(HttpRequest):
    htmx: HtmxDetails

CONNECTED_URL_SUCCESSFUL = None

fastapi_service_obj = None
# Get the first FastAPI Endpoint object
try:
    fastapi_service_obj = FastAPIEndpoint.objects.first()
except Exception as errr:
    print(f'Error occurred getting FastAPI Endpoint object: {errr}')

if fastapi_service_obj:
    # Get the FastAPI URL
    fastapi_detail = get_fastapi_url(fastapi_service_obj)
    fastapi_url: str = fastapi_detail.get('http_url', None)
    fastapi_verify_ssl: bool = fastapi_detail.get('verify_ssl', True)

def sync_resource(request: HtmxHttpRequest, path: str, template_name: str) -> HttpResponse:
    global CONNECTED_URL_SUCCESSFUL

    fastapi_path: str = f'{fastapi_url}/{path}' if fastapi_url else None

    if not fastapi_url:
        return HttpResponse(status=404, content='No FastAPI URL found')

    def make_request():
        try:
            response = requests.get(fastapi_path, verify=fastapi_verify_ssl)
            print(f'FastAPI response: {response.json()}')
            response.raise_for_status()
            CONNECTED_URL_SUCCESSFUL = fastapi_url
        except Exception as errr:
            print(f'Error occurred: {errr}')

            # Try to connect to FastAPI using the IP address and port.
            print(f'Trying to connect to FastAPI using the IP address and port: {fastapi_detail.get("ip_address_url")}')
            response = requests.get(fastapi_detail.get('ip_address_url') + f'/{path}', verify=False)
            print(f'FastAPI response: {response.json()}')
            CONNECTED_URL_SUCCESSFUL = fastapi_detail.get('ip_address_url')

    # Run the request in a separate thread
    Thread(target=make_request).start()

    return render(request, template_name)

@require_GET
def sync_devices(request: HtmxHttpRequest) -> HttpResponse:
    return sync_resource(request, 'dcim/devices/create', 'netbox_proxbox/sync_devices.html')

@require_GET
def sync_virtual_machines(request: HtmxHttpRequest) -> HttpResponse:
    return sync_resource(request, 'virtualization/virtual-machines/create', 'netbox_proxbox/sync_virtual_machines.html')

@require_GET
def sync_full_update(request: HtmxHttpRequest) -> HttpResponse:
    return sync_resource(request, 'full-update', 'netbox_proxbox/sync_full_update.html')


from django.urls import reverse
from django.db import models
from django.utils.translation import gettext_lazy as _
from django.core.validators import MaxValueValidator, MinValueValidator 

from netbox.models import NetBoxModel

from .fields import DomainField
from .choices import ProxmoxModeChoices, SyncTypeChoices, SyncStatusChoices

class ProxmoxEndpoint(NetBoxModel):
    name = models.CharField(
        default='Proxmox Endpoint',
        max_length=255,
        blank=True,
        null=True,
        help_text=_('Name of the Proxmox Endpoint/Cluster. It will be filled automatically by API.'),
    )
    ip_address = models.ForeignKey(
        to='ipam.IPAddress',
        on_delete=models.PROTECT,
        related_name='+',
        verbose_name=_('IP Address'),
        null=True,
        blank=True,
        help_text=_('IP Address of the Proxmox Endpoint (Cluster). Fallback if domain name is not provided.'),
    )
    domain = DomainField(
        verbose_name=_('Domain'),
        help_text=_('Domain name of the Proxmox Endpoint (Cluster).'),
        blank=True,
        null=True,
    )
    port = models.PositiveIntegerField(
        default=8006,
        validators=[MinValueValidator(1), MaxValueValidator(65535)],
        verbose_name=_('HTTP Port'),
    )
    mode = models.CharField(
        max_length=255,
        choices=ProxmoxModeChoices,
        default=ProxmoxModeChoices.PROXMOX_MODE_UNDEFINED,

    )
    version = models.CharField(max_length=20, blank=True, null=True)
    repoid = models.CharField(
        max_length=16,
        blank=True,
        null=True,
        verbose_name=_('Repository ID'),
    )
    username = models.CharField(
        default='root@pam',
        max_length=255,
        verbose_name=_('Username'),
        help_text=_("Username must be in the format of 'user@realm'. Default is 'root@pam'.")
    )
    password = models.CharField(
        max_length=255,
        verbose_name=_('Password'),
        help_text=_('Password of the Proxmox Endpoint. It is not needed if you use Token.'),
        blank=True,
        null=True,
    )
    token_name = models.CharField(
        max_length=255,
        verbose_name=_('Token Name'),
    )
    token_value = models.CharField(
        max_length=255,
        verbose_name=_('Token Value'),
    )
    verify_ssl = models.BooleanField(
        default=False,
        verbose_name=_('Verify SSL'),
        help_text=_('Choose or not to verify SSL certificate of the Proxmox Endpoint'),
    )

    class Meta:
        verbose_name_plural: str = "Proxmox Endpoints"
        unique_together = ['name', 'ip_address', 'domain']
        ordering = ('name',)
        
    def __str__(self):
        return f"{self.name} ({self.ip_address})"
    
    def get_absolute_url(self):
        return reverse('plugins:netbox_proxbox:proxmoxendpoint', args=[self.pk])


class NetBoxEndpoint(NetBoxModel):
    name = models.CharField(
        default='NetBox Endpoint',
        max_length=255,
        blank=True,
        null=True,
        help_text=_('Name of the NetBox Endpoint.'),
    )
    ip_address = models.ForeignKey(
        to='ipam.IPAddress',
        on_delete=models.PROTECT,
        related_name='+',
        verbose_name=_('IP Address'),
        null=True,
        blank=True,
        help_text=_('IP Address of the NetBox API. Fallback if domain name is not provided.'),
    )
    domain = DomainField(
        default='localhost',
        verbose_name=_('Domain'),
        help_text=_('Domain name of the NetBox API. Default is "localhost".'),
    )
    port = models.PositiveIntegerField(
        default=443,
        validators=[MinValueValidator(1), MaxValueValidator(65535)],
        verbose_name=_('HTTP Port'),
    )
    token = models.ForeignKey(
        to='users.Token',
        on_delete=models.PROTECT,
        related_name='+',
        verbose_name=_('API Token'),
        null=True,
        blank=True,
        help_text=_('API Token for the NetBox API. Needed for Proxbox Backend Service to communicate with NetBox.'),
    )
    verify_ssl = models.BooleanField(
        default=True,
        verbose_name=_('Verify SSL'),
        help_text=_('Choose or not to verify SSL certificate of the Netbox Endpoint'),
    )

    class Meta:
        verbose_name_plural: str = 'Netbox Endpoints'
        unique_together = ['name', 'ip_address']
        
    def __str__(self):
        return f"{self.name} ({self.ip_address})"

    def get_absolute_url(self):
        return reverse("plugins:netbox_proxbox:netboxendpoint", args=[self.pk])
        

class FastAPIEndpoint(NetBoxModel):
    name = models.CharField(
        default='ProxBox Endpoint',
        max_length=255,
        blank=True,
        null=True,
        help_text=_('Name of the ProxBox Endpoint.'),
    )
    ip_address = models.ForeignKey(
        to='ipam.IPAddress',
        on_delete=models.PROTECT,
        related_name='+',
        verbose_name=_('IP Address'),
        null=True,
        blank=True,
        help_text=_('IP Address of the Proxbox API (Backend Service). Fallback if domain name is not provided.'),
    )
    domain = DomainField(
        default='localhost',
        verbose_name=_('Domain'),
        help_text=_('Domain name of the Proxbox API (Backend Service). Default is "localhost".'),
    )
    port = models.PositiveIntegerField(
        default=8800,
        validators=[MinValueValidator(1), MaxValueValidator(65535)],
        verbose_name=_('HTTP Port'),
    )
    verify_ssl = models.BooleanField(
        default=True,
        verbose_name=_('Verify SSL'),
        help_text=_('Choose or not to verify SSL certificate of the Proxbox Endpoint'),
    )
    token = models.CharField(
        blank=True,
        null=True,
        max_length=255,
        verbose_name=_('Token'),
        help_text=_('Token for the Proxbox Endpoint. If not provided, the Proxbox Endpoint will not be able to send messages to the client (user) browser.'),
    )
    use_websocket = models.BooleanField(
        default=False,
        verbose_name=_('Use WebSocket'),
        help_text=_('Choose or not to use WebSocket for the Proxbox Endpoint. If enabled, the Proxbox Endpoint will use WebSocket connection to send messages to the client (user) browser.'),
    )
    websocket_domain = models.CharField(
        max_length=255,
        blank=True,
        null=True,
        verbose_name=_('WebSocket Domain'),
        help_text=_('Domain name of the WebSocket for the Proxbox Endpoint'),
    )
    websocket_port = models.PositiveIntegerField(
        default=8800,
        validators=[MinValueValidator(1), MaxValueValidator(65535)],
        verbose_name=_('WebSocket Port'),
        help_text=_('Port of the WebSocket for the Proxbox Endpoint (the same as HTTP port)'),
    )
    server_side_websocket = models.BooleanField(
        default=False,
        verbose_name=_('[BETA] Server Side WebSocket'),
        help_text=_('Choose or not to use server side WebSocket connection for the Proxbox Endpoint. This is experimental feature and may not work as expected. This way, client (user) browser will not be able to send messages to the Proxbox Endpoint.'),
    )

    class Meta:
        verbose_name_plural: str = 'FastAPI Endpoints'
        unique_together = ['name', 'ip_address']
    
    def __str__(self):
        return f"{self.name} ({self.domain})"

    def get_absolute_url(self):
        return reverse("plugins:netbox_proxbox:fastapiendpoint", args=[self.pk])


class SyncProcess(NetBoxModel):
    name = models.CharField(max_length=255, unique=True)
    sync_type = models.CharField(
        max_length=20,
        choices=SyncTypeChoices,
        default=SyncTypeChoices.ALL,
    )
    status = models.CharField(
        max_length=20,
        choices=SyncStatusChoices,
        default=SyncStatusChoices.NOT_STARTED,
    )  
    started_at = models.DateTimeField(
        null=True, 
        blank=True,
        help_text=_('When the sync process started. Format: YYYY-MM-DD HH:MM:SS')
    )
    completed_at = models.DateTimeField(
        null=True, 
        blank=True,
        help_text=_('When the sync process completed. Format: YYYY-MM-DD HH:MM:SS')
    )
    runtime = models.FloatField(
        null=True,
        blank=True,
        help_text=_('Time elapsed for the sync process. Format: seconds')
    )

    def __str__(self):
        return f'{self.name} ({self.sync_type})'
    
    def get_absolute_url(self):
        return reverse("plugins:netbox_proxbox:syncprocess", args=[self.pk])


from django.utils.translation import gettext_lazy as _
from utilities.choices import ChoiceSet

class ProxmoxModeChoices(ChoiceSet):
    key = 'ProxmoxEndpoint.mode'
    
    PROXMOX_MODE_UNDEFINED = 'undefined'
    PROXMOX_MODE_STANDALONE = 'standalone'
    PROXMOX_MODE_CLUSTER = 'cluster'
    
    CHOICES = [
        (PROXMOX_MODE_UNDEFINED, _('Undefined'), 'gray'),
        (PROXMOX_MODE_STANDALONE, _('Standalone'), 'blue'),
        (PROXMOX_MODE_CLUSTER, _('Cluster'), 'green'),
    ]
    
class SyncTypeChoices(ChoiceSet):
    key = 'SyncProcess.sync_type'
    
    VIRTUAL_MACHINES = 'virtual-machines'
    DEVICES = 'devices'
    ALL = 'all'
    
    CHOICES = [
        (VIRTUAL_MACHINES, _('Virtual Machines'), 'blue'),
        (DEVICES, _('Devices'), 'green'),
        (ALL, _('All'), 'red'),
    ]

class SyncStatusChoices(ChoiceSet):
    key = 'SyncProcess.status'
    
    NOT_STARTED = 'not-started'
    SYNCING = 'syncing'
    COMPLETED = 'completed'
    
    CHOICES = [
        (NOT_STARTED, _('Not Started'), 'gray'),
        (SYNCING, _('Syncing'), 'blue'),
        (COMPLETED, _('Completed'), 'green'),
    ]



from rest_framework import serializers

from netbox.api.serializers import NetBoxModelSerializer
from ipam.api.serializers import IPAddressSerializer
from ..models import ProxmoxEndpoint, NetBoxEndpoint, FastAPIEndpoint, SyncProcess
from ..choices import SyncTypeChoices, SyncStatusChoices
from extras.models import Tag

class SyncProcessSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_proxbox-api:syncprocess-detail',
    )
    sync_type = serializers.ChoiceField(choices=SyncTypeChoices)
    status = serializers.ChoiceField(choices=SyncStatusChoices)
    runtime = serializers.FloatField(required=False, allow_null=True)
    started_at = serializers.DateTimeField()
    completed_at = serializers.DateTimeField(required=False, allow_null=True)
    tags = serializers.PrimaryKeyRelatedField(queryset=Tag.objects.all(), many=True)
    
    class Meta:
        model = SyncProcess
        fields = (
            'id', 'url', 'display', 'name', 'sync_type', 'status', 'started_at', 'completed_at',
            'runtime', 'tags', 'created', 'last_updated',
        )

class ProxmoxEndpointSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_proxbox-api:proxmoxendpoint-detail',
    )
    ip_address = IPAddressSerializer()
    domain = serializers.CharField(required=False, allow_null=True)
    
    class Meta:
        model = ProxmoxEndpoint
        fields = (
            'id', 'url', 'display', 'name', 'ip_address', 'domain', 'port',
            'token_name', 'token_value', 'username', 'password', 'verify_ssl',
            'mode', 'version', 'repoid', 
            'tags', 'custom_fields', 'created', 'last_updated',
        )


class NetBoxEndpointSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_proxbox-api:netboxendpoint-detail',
    )
    ip_address = IPAddressSerializer(nested=True, required=False, allow_null=True)
    
    class Meta:
        model = NetBoxEndpoint
        fields = (
            'id', 'url', 'display', 'name', 'ip_address', 'port',
            'token', 'verify_ssl', 'tags', 'custom_fields',
            'created', 'last_updated',
        )
    
    
    def create(self, validated_data):
        """
        Check if a NetBoxEndpoint already exists. If it does, return the existing one.
        Otherwise, create a new one.
        """
        if NetBoxEndpoint.objects.exists():
            return NetBoxEndpoint.objects.first()
        return super().create(validated_data)
    

class FastAPIEndpointSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_proxbox-api:fastapiendpoint-detail',
    )
    ip_address = IPAddressSerializer(nested=True, required=False, allow_null=True)
    
    class Meta:
        model = FastAPIEndpoint
        fields = (
            'id', 'url', 'display', 'name', 'ip_address', 'port',
            'verify_ssl', 'tags', 'custom_fields', 'created', 'last_updated',
        )

    def create(self, validated_data):
        """
        Check if a FastAPIEndpoint already exists. If it does, return the existing one.
        Otherwise, create a new one.
        """
        if FastAPIEndpoint.objects.exists():
            return FastAPIEndpoint.objects.first()
        return super().create(validated_data)


# Django Imports
from django import forms

# NetBox Imports
from netbox.forms import NetBoxModelForm, NetBoxModelFilterSetForm
from utilities.forms.fields import CommentField, DynamicModelChoiceField
from ipam.models import IPAddress
from django.utils.translation import gettext as _

# Proxbox Imports
from ..models import ProxmoxEndpoint
from ..choices import ProxmoxModeChoices


class ProxmoxEndpointForm(NetBoxModelForm):
    """
    Form for ProxmoxEndpoint model.
    It is used to CREATE and UPDATE ProxmoxEndpoint objects.
    """
    ip_address = DynamicModelChoiceField(
        queryset=IPAddress.objects.all(),
        help_text=_('Select a NetBox IP Address'),
        label=_('IP Address'),
        required=False
    )
    domain = forms.CharField(
        required=False,
        help_text=_('Domain name of the Proxmox Endpoint (Cluster). It will try using the DNS name provided in IP Address if it is not empty.'),
        label=_('Domain')
    )
    verify_ssl = forms.BooleanField(
        required=False,
        help_text=_('Choose or not to verify SSL certificate of the Proxmox Endpoint. Only use this if you are sure about the SSL certificate of the Proxmox Endpoint.'),
        label=_('Verify SSL')
    )
    comments = CommentField()
    
    class Meta:
        model = ProxmoxEndpoint
        fields = (
            'name', 'ip_address', 'domain', 'port', 'username',
            'password', 'token_name', 'token_value', 'verify_ssl',
            'tags'
        )


class ProxmoxEndpointFilterForm(NetBoxModelFilterSetForm):
    """
    Filter form for ProxmoxEndpoint model.
    It is used in the ProxmoxEndpointListView.
    """
    
    model = ProxmoxEndpoint
    name = forms.CharField(
        required=False
    )
    ip_address = forms.ModelMultipleChoiceField(
        queryset=IPAddress.objects.all(),
        required=False,
        help_text='Select IP Address'
    )
    mode = forms.MultipleChoiceField(
        choices=ProxmoxModeChoices,
        required=False
    )



# Generated by Django 5.1.4 on 2025-01-17 21:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_proxbox', '0004_fastapiendpoint_netboxendpoint_proxmoxendpoint_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='proxmoxendpoint',
            name='name',
            field=models.CharField(blank=True, default='Proxmox Endpoint', max_length=255, null=True),
        ),
    ]



from django.urls import include, path
from utilities.urls import get_model_urls

from netbox_proxbox.websocket_client import websocket_client, WebSocketView

from netbox.views.generic import ObjectChangeLogView

from . import models, views

urlpatterns = [
    # Home View
    path('', views.HomeView.as_view(), name='home'),
    path('nodes', views.NodesView.as_view(), name='nodes'),
    path('virtual_machines', views.VirtualMachinesView.as_view(), name='virtual_machines'),
    path('contributing/', views.ContributingView.as_view(), name='contributing'),
    path('community/', views.CommunityView.as_view(), name='community'),
    
    path('fix-proxbox-backend/', views.FixProxboxBackendView.as_view(), name='fix-proxbox-backend'),
    path('start-proxbox-backend/', views.FixProxboxBackendView.as_view(), name='start-proxbox-backend'),
    path('stop-proxbox-backend/', views.StopProxboxBackendView.as_view(), name='stop-proxbox-backend'),
    path('restart-proxbox-backend/', views.RestartProxboxBackendView.as_view(), name='restart-proxbox-backend'),
    path('status-proxbox-backend/', views.StatusProxboxBackendView.as_view(), name='status-proxbox-backend'),

    # Redirect to: "https://github.com/orgs/netdevopsbr/discussions"
    path('discussions/', views.DiscussionsView, name='discussions'),
    path('discord/', views.DiscordView, name='discord'),
    path('telegram/', views.TelegramView, name='telegram'),
    
    # ProxmoxEndpoint Model URLs
    path('endpoints/proxmox/', views.ProxmoxEndpointListView.as_view(), name='proxmoxendpoint_list'),
    path('endpoints/proxmox/add/', views.ProxmoxEndpointEditView.as_view(), name='proxmoxendpoint_add'),
    path('endpoints/proxmox/<int:pk>', views.ProxmoxEndpointView.as_view(), name='proxmoxendpoint'),
    path('endpoints/proxmox/<int:pk>/edit/', views.ProxmoxEndpointEditView.as_view(), name='proxmoxendpoint_edit'),
    path('endpoints/proxmox/<int:pk>/delete/', views.ProxmoxEndpointDeleteView.as_view(), name='proxmoxendpoint_delete'),
    path('endpoints/proxmox/<int:pk>/changelog/', ObjectChangeLogView.as_view(), name='proxmoxendpoint_changelog', kwargs={
        'model': models.ProxmoxEndpoint
    }),
    
    # NetBoxEndpoint Model URLs
    path('endpoints/netbox/', views.NetBoxEndpointListView.as_view(), name='netboxendpoint_list'),
    path('endpoints/netbox/add/', views.NetBoxEndpointEditView.as_view(), name='netboxendpoint_add'),
    path('endpoints/netbox/<int:pk>', views.NetBoxEndpointView.as_view(), name='netboxendpoint'),
    path('endpoints/netbox/<int:pk>/edit/', views.NetBoxEndpointEditView.as_view(), name='netboxendpoint_edit'),
    path('endpoints/netbox/<int:pk>/delete/', views.NetBoxEndpointDeleteView.as_view(), name='netboxendpoint_delete'),
    path('endpoints/netbox/<int:pk>/changelog/', ObjectChangeLogView.as_view(), name='netboxendpoint_changelog', kwargs={
        'model': models.NetBoxEndpoint
    }),
    
    
    # FastAPIEndpoint Model URLs
    path('endpoints/fastapi/', views.FastAPIEndpointListView.as_view(), name='fastapiendpoint_list'),
    path('endpoints/fastapi/add/', views.FastAPIEndpointEditView.as_view(), name='fastapiendpoint_add'),
    path('endpoints/fastapi/<int:pk>', views.FastAPIEndpointView.as_view(), name='fastapiendpoint'),
    path('endpoints/fastapi/<int:pk>/edit/', views.FastAPIEndpointEditView.as_view(), name='fastapiendpoint_edit'),
    path('endpoints/fastapi/<int:pk>/delete/', views.FastAPIEndpointDeleteView.as_view(), name='fastapiendpoint_delete'),
    path('endpoints/fastapi/<int:pk>/changelog/', ObjectChangeLogView.as_view(), name='fastapiendpoint_changelog', kwargs={
        'model': models.FastAPIEndpoint
    }),
    
    # SyncProcess Model URLs
    path('sync-processes/', views.SyncProcessListView.as_view(), name='syncprocess_list'),
    path('sync-processes/add/', views.SyncProcessEditView.as_view(), name='syncprocess_add'),
    path('sync-processes/<int:pk>', views.SyncProcessView.as_view(), name='syncprocess'),
    path('sync-processes/<int:pk>/edit/', views.SyncProcessEditView.as_view(), name='syncprocess_edit'),
    path('sync-processes/<int:pk>/delete/', views.SyncProcessDeleteView.as_view(), name='syncprocess_delete'),
    path('sync-processes/<int:pk>/changelog/', ObjectChangeLogView.as_view(), name='syncprocess_changelog', kwargs={
        'model': models.SyncProcess
    }),
    
    # Manual Sync (HTTP Request)
    path('sync/devices', views.sync_devices, name='sync_devices'),
    path('sync/virtual-machines', views.sync_virtual_machines, name='sync_virtual_machines'),
    path('sync/full-update', views.sync_full_update, name='sync_full_update'),
    
    path('keepalive-status/<str:service>/<int:pk>', views.get_service_status, name='keepalive_status'),
    path('proxmox-card/<int:pk>', views.get_proxmox_card, name='proxmox_card'),
    path('websocket/<str:message>', WebSocketView.as_view(), name='websocket'),
    
]